const QuestApi = require('../net/QuestApi');
const DWTool = require("../utils/DWTool");
const QuestDailyMissionType = require("../utils/GameEnum").QuestDailyMissionType;

cc.Class({
    extends: cc.Component,

    properties: {
        questWrap: {
            tooltip: '每日任务容器',
            default: null,
            type: cc.Node
        },

        questBtns: [cc.Node],

        userDegreeRichText: {
            tooltip: '用户当前活跃度',
            default: null,
            type: cc.RichText
        },
        _userDegree: 0,
        userDegree: {
            get () {
                return this._userDegree;
            },
            set (value) {
                this._userDegree = value;

                // 设置用户当前活跃度
                // this.actProgress.progress = data / 100
                // this.userDegreeLabel.string = data
            }
        },

        giftFrames: {
            tooltip: '奖励图片素材',
            default: [],
            type: [cc.SpriteFrame]
        },

        questItem: cc.Prefab,
        resetTimeLabel: cc.Label,

        isLoadData: {
            default: false,
            visible: false
        }
    },


    onLoad () {
        this.userDegree = 0;
        this.dailyDegreeList = [];

        this.giftMap = ['diamond', 'coin', 'ticket'];
    },

    init (quest) {
        this.quest = quest;
        this.loadNetworkData();
    },

    loadNetworkData() {
        QuestApi.getDailyTask((responseData) => {
            this.dailyDegreeList = responseData.dailyDegreeList.sort((a, b) => {
                return a.count - b.count
            });

            this.dailyTaskList = responseData.dailyTaskList;
            this.userDegree = parseInt(responseData.userDegree);

            this._setUserDegree();
            this._setTaskList();
            this.updateTime = responseData.updateTime;
            this.isLoadData = true;
        }   , (error) => {
            console.log('dailytask error' + error);
        });
    },

    /**
     * 渲染每日任务列表
     */
    _setTaskList () {

        this.questWrap.removeAllChildren();

        this.dailyTaskList.forEach((item, index) => {
            let questItem = cc.instantiate(this.questItem);
            this.questWrap.addChild(questItem);
            questItem.getComponent('QuestDailyItem').init(this, item, index);
        })
    },

    /**
     * 设置活跃度奖励进度条
     */
    _setUserDegree () {
        var canGet = false;
        var alreadyGetCount = 0;

        for (let i = 0; i < this.dailyDegreeList.length; ++i) {
            let item = this.dailyDegreeList[i];
            if (item.state == QuestDailyMissionType.CanGain) {
                canGet = true;
                this.degree = item;
                break;
            } else if (item.state == QuestDailyMissionType.AlreadyGet){
                alreadyGetCount++;
            }
        }

        let taskCount= this.dailyDegreeList.length;
        this.userDegreeRichText.string = `<b><outline color=#2E2E2E width=2>每日任务 最终奖励 </outline></b><size=18><outline color=#2E2E2E width=2>(${this.userDegree}/${taskCount})</outline></size>`;

        this.questBtns.forEach(n => {
            n.active = false;
        });

        let btn;
        if (canGet) {
            btn = this.questBtns[1];
        } else if (alreadyGetCount == taskCount) {
            btn = this.questBtns[2];
        } else {
            for (let i = 0; i < this.dailyDegreeList.length; ++i) {
                let item = this.dailyDegreeList[i];
                if (item.state == QuestDailyMissionType.NoFinished){
                    this.degree = item;
                    break;
                }
            }
            btn = this.questBtns[0];
        }
        btn.active = true;

        let countNode = cc.find('/awardNode/count_label', btn);
        let spriteNode = cc.find('/awardNode/sprite', btn);
        let awardLabelNode = cc.find('/awardLabel', btn);

        var awardString = '奖励';
        if (canGet) {
            awardString = '获得';
        }

        let _count = 0;
        let _type = '';
        let _sprite = null;

        this.giftMap.forEach((value, index) => {
            if(this.degree[value] > 0) {
                _count = this.degree[value];
                _type = value;
                _sprite = this.giftFrames[index];
            }
        });

        spriteNode.getComponent('cc.Sprite').spriteFrame = _sprite;
        countNode.getComponent('cc.Label').string = `${DWTool.coinParseNoFixed(_count)}`;

        awardLabelNode.getComponent('cc.Label').string = awardString;

    },

    /**
     * 领取活跃度奖励
     */
    handleGiftBtn () {
        // state = 0, 可领取状态
        if(this.degree.state == QuestDailyMissionType.CanGain) {

            QuestApi.postGetDailyDegreeReward(this.degree.count, (res) => {
                // 更新全局userInfo
                this.quest.updateUserInfo(this.degree.coin, this.degree.diamond, this.degree.ticket);

                // state = 2, 已领取状态
                this.degree.state = QuestDailyMissionType.AlreadyGet;

                // 显示动画
                setTimeout(() => {
                    this.quest.showActGift({
                        ticket: this.degree.ticket,
                        diamond: this.degree.diamond,
                        coin: this.degree.coin
                    })
                }, 200);
                this._setUserDegree();

            }, (error) => {

            });
        }
    },

    update(dt) {
        
        if (this.updateTime == undefined) {
            return;
        }
        
        if (this.isLoadData == true) {
            var curTimestamp = parseInt(new Date().getTime());
            if ((this.updateTime - curTimestamp) > 0) {
                this.resetTimeLabel.string = `每日任务重置：${DWTool.calculateTimeString((this.updateTime - curTimestamp) / 1000)}`;
            } else {
                this.isLoadData = false;
                this.loadNetworkData();
            }
        }
    }

    // start () {
    //
    // }
});
